package lightcontainer.domains.client;

import lightcontainer.storage.AppData;
import lightcontainer.storage.Repository;
import lightcontainer.storage.User;
import lightcontainer.utils.AES_GCM;
import lightcontainer.utils.ShaHasher;

import java.security.NoSuchAlgorithmException;
import java.util.LinkedList;

/**
 * Contexte associé à la requête d'un utilisateur.
 * Permet de récupérer les données à celui-ci et d'effectuer des actions sur le context courant.
 */
public class Context {

    private Repository repository;


    /**
     * Login de l'utilisateur
     */
    private String login;

    // Constructeur
    public Context(Repository repository) {
        this.repository = repository;
    }


    /**
     * Permet de créer un utilisateur.
     *
     * @param login    Login de l'utilisateur
     * @param password Mot de passe de l'utilisateur
     * @return TRUE si l'utilisateur a pu être créé
     */
    public boolean createUser(String login, String password) {
        try {
            String key = AES_GCM.generateSecretKey();

            ShaHasher hasher = new ShaHasher(password);
            password = hasher.nextHashing();
            String passwordSalt = hasher.getSalt();

            if (this.repository.addUser(login, password, key, passwordSalt)) {
                this.login = login;
                return true;
            }
        } catch (AES_GCM.AesGcmException e) {
        }
        return false;
    }

    /**
     * Login de l'utilisateur
     *
     * @return Login de l'utilisateur
     */
    public String getLogin() {
        return this.login;
    }


    /**
     * Permet de demander la connection de l'utilisateur
     *
     * @param login    Login
     * @param password Mot de passe
     * @return TRUE si l'utilisateur a été authentifié
     */
    public boolean signIn(String login, String password) {
        String passwordSalt = this.repository.getUserPasswordSalt(login);
        if (passwordSalt != null) {
            ShaHasher hasher = new ShaHasher(password);

            if (this.repository.verifyUser(login, hasher.fromSalt(hasher.saltToByte(passwordSalt)))) {
                this.login = login;
                return true;
            }
        }
        return false;
    }

    /**
     * Permet de savoir si l'utilisateur s'est connecté (Avec login et mdp)
     *
     * @return TRUE si l'utilisateur est connecté
     */
    public boolean isConnected() {
        return this.getLogin() != null;
    }


    /**
     * Clé AES de l'utilisateur
     * @return Clé AES de l'utilisateur
     */
    public String getAesKey() {
        return this.repository.getUserAesKey(getLogin());
    }
}
